"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MeshTool = void 0;
/*
 * @Description  :
 * @Author       : 赵耀圣
 * @Q群           : 632839661
 * @Date         : 2020-12-10 15:01:42
 * @LastEditTime : 2021-03-11 09:09:57
 * @FilePath     : \cga.js\src\render\mesh.ts
 */
var Vec3_1 = require("../math/Vec3");
var array_1 = require("../utils/array");
var geometry_1 = require("./geometry");
var trianglution_1 = require("../alg/trianglution");
/**
 * 网格工具
 */
var MeshTool = /** @class */ (function () {
    function MeshTool() {
    }
    MeshTool.indexable = function (obj, refIndexInfo, force) {
        if (refIndexInfo === void 0) { refIndexInfo = { index: 0 }; }
        if (force === void 0) { force = false; }
        if (obj instanceof Array) {
            for (var i = 0; i < obj.length; i++)
                MeshTool.indexable(obj[i], refIndexInfo);
        }
        else if (obj instanceof Object) {
            if (obj.index === undefined)
                obj.index = refIndexInfo.index++;
            else if (force)
                obj.index = refIndexInfo.index++;
        }
    };
    MeshTool.triangListToBuffer = function (vertices, triangleList) {
        MeshTool.indexable(triangleList);
        var indices = [];
        (0, array_1.forall)(triangleList, function (v) {
            indices.push(v.index);
        });
        return MeshTool.toGeoBuffer(vertices, indices);
    };
    /**
     * 顶点纹理坐标所以转化为buffer数据
     * @param {Array<Verctor3|Number>} vertices
     * @param {Array<Number>} indices
     * @param {Array<Verctor2|Number>} uvs
     */
    MeshTool.toGeoBuffer = function (vertices, indices, uvs) {
        var geometry = new geometry_1.BufferGeometry();
        geometry.addAttribute('position', vertices, 3);
        geometry.addAttribute('uv', uvs || new Float32Array(geometry.getAttribute('position').array.length / 3 * 2), 2);
        geometry.setIndex(indices);
        return geometry;
    };
    /**
     * 三角剖分后转成几何体
     * 只考虑XY平面
     * @param {*} boundary
     * @param {*} hole
     * @param {*} options
     */
    MeshTool.trianglutionToGeometryBuffer = function (boundary, holes, options) {
        if (holes === void 0) { holes = []; }
        if (options === void 0) { options = { normal: Vec3_1.Vec3.UnitZ }; }
        var triangles = (0, trianglution_1.triangulation)(boundary, holes, options);
        var vertices = __spreadArray(__spreadArray([], boundary, true), (0, array_1.flat)(holes), true);
        var uvs = [];
        vertices.forEach(function (v) {
            uvs.push(v.x, v.z);
        });
        var geometry = MeshTool.toGeoBuffer(vertices, triangles, uvs);
        return geometry;
    };
    return MeshTool;
}());
exports.MeshTool = MeshTool;
